/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2010, Kyoto University and Honda Motor Co.,Ltd. All rights reserved.
 *
 * HARK was developed by researchers in Okuno Laboratory at the Kyoto University and 
 * Honda Research Institute Japan Co.,Ltd.
 *
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this list 
 *    of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice, this 
 *    list of conditions and the following disclaimer in the documentation and/or other 
 *    materials provided with the distribution.
 *  * Neither the name of Kyoto University and Honda Motor Co.,Ltd., Inc. nor the names 
 *    of its contributors may be used to endorse or promote products derived from this 
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCI-
 * DENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSI-
 * NESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTR-
 * ACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "BufferedNode.h"
#include "operators.h"
#include "CvfdImage.h"
#include "Map.h"
#include "Vector.h"
#include "CvfdCameraSpec.h"

using namespace std;
using namespace FD;
  
class CvfdhCameraSpecSetter;
  
DECLARE_NODE(CvfdhCameraSpecSetter)
/*Node
 *
 * @name CvfdhCameraSpecSetter
 * @category FDOPENCV_HARK:Misc
 * @description Set CameraSpec of the input image such as view angles, etc.
 *
 * @parameter_name VIEW_ANGLE
 * @parameter_type object
 * @parameter_value <Vector<float> 40.0 30.0>
 * @parameter_description Vertical and horizontal view angle of the camera
 *
 * @parameter_name TRANS_OFFSET
 * @parameter_type object
 * @parameter_value <Vector<float> 0.0 0.0>
 * @parameter_description Vertical and horizontal offset for image transformation
 *
 * @parameter_name TRANS_OFFSET_SCALE
 * @parameter_type object
 * @parameter_value <Vector<float> 1.0 1.0>
 * @parameter_description Vertical and horizontal offset scale for image transformation
 *
 * @parameter_name RADIUS_SCALE
 * @parameter_type float
 * @parameter_value 1.0
 * @parameter_description Radius scale term for the image transformation
 *
 * @parameter_name CAMERA_POS
 * @parameter_type object
 * @parameter_value <Vector<float> 0.0 0.0 0.0>
 * @parameter_description Camera position from the base
 *
 * @input_name IMAGEIN
 * @input_type CvfdImage
 * @input_description Input image. The type should be CvfdImage.
 *
 * @output_name CAMSPEC
 * @output_description Common specification of the camera (CvfdCameraSpec) 
 * @output_type CvfdCameraSpec
 *
 END*/
    
class CvfdhCameraSpecSetter : public BufferedNode {
    
  int m_imageInID;
  int m_camspecOutID; 

  int img_width;
  int img_height;
  int img_channels;  

  Vector<float> view_angle;
  Vector<float> trans_offset;
  Vector<float> trans_offset_scale;
  float radius_scale;
  Vector<float> camera_pos;
    
public:
  CvfdhCameraSpecSetter(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  {
    m_imageInID = addInput("IMAGEIN");
    m_camspecOutID = addOutput("CAMSPEC");

    img_width    = 0;
    img_height   = 0;
    img_channels = 1;
    
    view_angle = object_cast<Vector<float> >(parameters.get("VIEW_ANGLE"));
    trans_offset = object_cast<Vector<float> >(parameters.get("TRANS_OFFSET"));
    trans_offset_scale = object_cast<Vector<float> >(parameters.get("TRANS_OFFSET_SCALE"));
    radius_scale = dereference_cast<float>(parameters.get("RADIUS_SCALE"));
    camera_pos = object_cast<Vector<float> >(parameters.get("CAMERA_POS"));

    inOrder = true;
  }
            
  void calculate(int output_id, int count, Buffer &out)
  {

    ObjectRef in = getInput(m_imageInID, count);
    RCPtr<CvfdImage> imagePtr = getInput(m_imageInID,count);   

    if(imagePtr->getImage() != NULL){
      IplImage* img_in;
      img_in = cvCloneImage(imagePtr->getImage());
      img_width = img_in->width;
      img_height = img_in->height;
      cvReleaseImage(&img_in);
    }
    
    CvfdCameraSpec* camspec = new CvfdCameraSpec();
    camspec->width = img_width;
    camspec->height = img_height;
    for(int cnt = 0; cnt < 2; cnt++) camspec->deg[cnt] = view_angle[cnt];
    for(int cnt = 0; cnt < 2; cnt++) camspec->offset[cnt] = trans_offset[cnt];
    for(int cnt = 0; cnt < 2; cnt++) camspec->offset_scale[cnt] = trans_offset_scale[cnt];
    camspec->scale_r = radius_scale;
    for(int cnt = 0; cnt < 3; cnt++) camspec->pos[cnt] = camera_pos[cnt];
    (*(outputs[m_camspecOutID].buffer))[count] = ObjectRef(camspec);      
   
  }
      
  NO_ORDER_NODE_SPEEDUP(CvfdhCameraSpecSetter)
};

